/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlBinop.h"
#include "HvSignalVar.h"
#include "HvControlIf.h"
#include "HvControlVar.h"
#include "HvControlCast.h"
#include "HvMath.h"
#include "HvSignalLine.h"
#include "HvSignalEnvelope.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_eJ99AEWI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_77nKYynB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_VXGx5Ieg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zXuPg2Sj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_d5rpiJPr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_9u1kTpDR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_9MiGlQvv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_3BdkcnNo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_dxwh0ikw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_uoSPDZjm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_LbzUAEqr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_nsNMFQ3L_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_iSvLvd3X_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_oqRHrSuG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_J0HwgbLI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_xAaBxX4s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_9LicYI6N_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_yQuqe8Jc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_WYFZlxAX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_jhl4RhBo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_aKdxPIvj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_97XFldHw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Nx0eKt3P_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_at9bRVx9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Vc4YXvQP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_HaaM9IFe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_o6Uw81sd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_okpSyGhC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_TKk9jVbY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_LMGLVSJY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_8gtdwIwR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_8E0bYVSC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_H2CEQmKr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_gxPWAwjH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_TBpqcju5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_YcHreAzF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_GDwCLELh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_pBOQPe2s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_E9S6lFS3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_grmW43um_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_0NSrfDrF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_CwNza3Yi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_s302qbiN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_DdVKbKMw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_IoCWFdWD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_HC14FT5q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_QS2znkY5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_50prOGTd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_J7bmPpXT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_opcwbYrG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_6rodcWom_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_LCBHCbNo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_xLioueAB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_bAywYPi9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_ZRtZJqqm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_lq4JnSMa_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_8zwZA9Rz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_671PxLce_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_3uFgSzGC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_GopZccTE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_DwZdenFV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_xOntdvcx_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_lfWI4sKP;
  SignalEnvelope sEnv_yQuqe8Jc;
  SignalEnvelope sEnv_WYFZlxAX;
  SignalEnvelope sEnv_TBpqcju5;
  SignalEnvelope sEnv_YcHreAzF;
  SignalLine sLine_Ag2EHKVS;
  SignalLine sLine_7aaseP74;
  ControlBinop cBinop_eJ99AEWI;
  ControlBinop cBinop_77nKYynB;
  ControlBinop cBinop_VXGx5Ieg;
  ControlBinop cBinop_zXuPg2Sj;
  ControlBinop cBinop_9MiGlQvv;
  ControlBinop cBinop_dxwh0ikw;
  ControlBinop cBinop_uoSPDZjm;
  ControlBinop cBinop_LbzUAEqr;
  ControlVar cVar_nsNMFQ3L;
  ControlVar cVar_oqRHrSuG;
  SignalVarf sVarf_EpQVDldP;
  ControlVar cVar_J0HwgbLI;
  ControlIf cIf_xAaBxX4s;
  ControlIf cIf_9LicYI6N;
  ControlBinop cBinop_jhl4RhBo;
  ControlBinop cBinop_97XFldHw;
  ControlBinop cBinop_Nx0eKt3P;
  ControlBinop cBinop_HaaM9IFe;
  ControlBinop cBinop_okpSyGhC;
  SignalVarf sVarf_G6CtLXoG;
  ControlVar cVar_8E0bYVSC;
  ControlIf cIf_H2CEQmKr;
  ControlIf cIf_gxPWAwjH;
  ControlBinop cBinop_GDwCLELh;
  ControlBinop cBinop_E9S6lFS3;
  ControlBinop cBinop_grmW43um;
  ControlBinop cBinop_s302qbiN;
  ControlBinop cBinop_IoCWFdWD;
  ControlVar cVar_J7bmPpXT;
  ControlBinop cBinop_opcwbYrG;
  SignalVarf sVarf_hxEacy2Q;
  SignalVarf sVarf_mjdVIAfo;
  SignalVarf sVarf_WpSec159;
  SignalVarf sVarf_BXukZeUz;
  ControlBinop cBinop_bAywYPi9;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
